﻿Type=Class
Version=7.01
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Custom View class 
'JustImgSlider - simple manual image slider, v.0.2
'(c) 26.05.2017 Peacemaker
'#Event: ExampleEvent (Value As Int)
'#DesignerProperty: Key: BooleanExample, DisplayName: Boolean Example, FieldType: Boolean, DefaultValue: True, Description: Example of a boolean property.
'#DesignerProperty: Key: IntExample, DisplayName: Int Example, FieldType: Int, DefaultValue: 10, MinRange: 0, MaxRange: 100, Description: Note that MinRange and MaxRange are optional.
'#DesignerProperty: Key: StringWithListExample, DisplayName: String With List, FieldType: String, DefaultValue: Sunday, List: Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday
'#DesignerProperty: Key: StringExample, DisplayName: String Example, FieldType: String, DefaultValue: Text
'#DesignerProperty: Key: ColorExample, DisplayName: Color Example, FieldType: Color, DefaultValue: 0xFFCFDCDC, Description: You can use the built-in color picker to find the color values.
'#DesignerProperty: Key: DefaultColorExample, DisplayName: Default Color Example, FieldType: Color, DefaultValue: Null, Description: Setting the default value to Null means that a nullable field will be displayed.
Sub Class_Globals
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Private mBase As Panel
	Private Const DefaultColorConstant As Int = -984833 'ignore
	Private lstSlides As List
	Private lblStatus As Label
	Type Slide (Name As String, URI As String, pnl As Panel)
	Private CurrentSlideID, NextID As Int
	Private PrevX As Int, LeftDirection As Boolean
End Sub

Public Sub Initialize (Callback As Object, EventName As String)
	mEventName = EventName
	mCallBack = Callback
	lstSlides.Initialize
End Sub

Public Sub DesignerCreateView (Base As Panel, lbl As Label, Props As Map)
	mBase = Base
    
	lstSlides.Initialize
	lblStatus.Initialize("")
	lblStatus.Text = "No images"
	lblStatus.Gravity = Bit.Or(Gravity.CENTER_HORIZONTAL, Gravity.CENTER_VERTICAL)
	lblStatus.TextColor = Colors.White
	If lblStatus.IsInitialized Then
		lblStatus.RemoveView
		mBase.AddView(lblStatus, (mBase.Width - mBase.Width)/2, mBase.Height - 20dip, mBase.Width, 20dip)
	End If
End Sub

Public Sub GetBase As Panel
	Return mBase
End Sub

#Region Code-manipulations
Public Sub AddToParent(Parent As Activity, Left As Int, Top As Int, Width As Int, Height As Int)
	mBase.Initialize("mBase")
	Parent.AddView(mBase, Left, Top, Width, Height)
End Sub

Public Sub getLeft As Int
	Return mBase.Left
End Sub
Public Sub setLeft(Left As Int)
	mBase.Left = Left
End Sub
Public Sub getTop As Int
	Return mBase.Top
End Sub
Public Sub setTop(Top As Int)
	mBase.Left = Top
End Sub
Public Sub getWidth As Int
	Return mBase.Width
End Sub
Public Sub setWidth(Width As Int)
	mBase.Width = Width
End Sub
Public Sub getHeight As Int
	Return mBase.Height
End Sub
Public Sub setHeight(Height As Int)
	mBase.Height = Height
End Sub
#End Region

'add an image URL
Public Sub AddSlide (Name As String, URL As String)
	Dim a As Slide
	a.Initialize
	a.Name = Name
	a.URI = URL
	
	Dim p As Panel
	p.Initialize("pnlSlide")
	mBase.AddView(p, 0, 0, mBase.Width, mBase.Height)
	p.SendToBack
	a.pnl = p
	a.pnl.Tag = lstSlides.Size	'record id for reference
	lstSlides.Add(a)
	DownloadImage(URL, p)
	Update_Status
End Sub

'add a local image file
Public Sub AddImageFile (Name As String, Folder As String, FileName As String)
	Dim a As Slide
	a.Initialize
	a.Name = Name
	a.URI = File.Combine(Folder, FileName)
	
	Dim p As Panel
	p.Initialize("pnlSlide")
	mBase.AddView(p, 0, 0, mBase.Width, mBase.Height)
	p.SendToBack
	a.pnl = p
	a.pnl.Tag = lstSlides.Size	'record id for reference
	lstSlides.Add(a)
	p.SetBackgroundImage(LoadBitmap(Folder, FileName))
	Update_Status
End Sub

'remove all slides
Public Sub Clear
	For i = 0 To lstSlides.Size - 1
		Dim a As Slide
		a = lstSlides.Get(i)
		a.pnl.SetBackgroundImage(Null)
		a.pnl.RemoveView
	Next
	lstSlides.Clear
	Update_Status
End Sub

Sub DownloadImage(Link As String, pnl As Panel)
	Dim j As HttpJob
	j.Initialize("", Me)
	j.Download(Link)
	Wait For (j) JobDone(j As HttpJob)
	If j.Success Then
		pnl.SetBackgroundImage(j.GetBitmap)
	End If
	j.Release
End Sub

Sub pnlSlide_Touch (Action As Int, X As Float, Y As Float)
	If lstSlides.Size <= 1 Then
		Return
	End If
	Dim pnl As Panel = Sender
	
	If Action = pnl.ACTION_DOWN Then
		PrevX = X
	Else If Action = pnl.ACTION_MOVE Then
		If X <= PrevX Then
			LeftDirection = True
		Else
			LeftDirection = False
		End If
		If pnl.Left < 0 Then	'next slide will be .....
			NextID = CurrentSlideID + 1
		Else
			NextID = CurrentSlideID - 1
		End If
		Check_Next
		Hide_others
		
		If Abs(X - PrevX) > 2%x Then
			If LeftDirection Then
				Dim delta As Int = pnl.Left - Abs(PrevX - X)/2
			Else
				Dim delta As Int = pnl.Left + Abs(PrevX - X)/2
			End If
			pnl.SetLayoutAnimated(500, delta, pnl.Top, pnl.Width, pnl.Height)
		End If
	Else If Action = pnl.ACTION_UP Then
		If pnl.Left <> 0 Then	'if moved, so taps are ignored
			If LeftDirection Then
				pnl.SetLayoutAnimated(500, -pnl.Left, pnl.Top, pnl.Width, pnl.Height)
			Else
				pnl.SetLayoutAnimated(500, mBase.Width + 1dip, pnl.Top, pnl.Width, pnl.Height)
			End If
			pnl.SendToBack
			pnl.Left = 0
			Check_Next
			Dim a As Slide = lstSlides.Get(NextID)	'next image
			Dim NextPnl As Panel = a.pnl
			NextPnl.BringToFront
			NextPnl.Left = 0
			
			CurrentSlideID = NextID
			Update_Status
			Dim a As Slide = lstSlides.Get(NextID)
			ToastMessageShow(a.Name, False)
		End If
	End If
End Sub

Private Sub Check_Next
	'next slide image
	If NextID > lstSlides.Size -1 Then
		NextID = 0
	End If
	If NextID < 0 Then
		NextID = lstSlides.Size - 1
	End If
End Sub

'set the next image under the current, and hide others
Private Sub Hide_others
	For i = 0 To lstSlides.Size - 1
		Dim a As Slide
		a = lstSlides.Get(i)
		If i = NextID Or i = CurrentSlideID Then
		Else
			a.pnl.SendToBack
		End If
	Next
End Sub

Private Sub Update_Status
	If lstSlides.Size > 0 Then
		If lstSlides.Size <= 5 Then
			Dim cs As CSBuilder
			cs.Initialize.Typeface(Typeface.FONTAWESOME)
			For i = 0 To lstSlides.Size - 1
				If i = CurrentSlideID Then
					cs.Append(Chr(0xF111))
				Else
					cs.Append(Chr(0xF10C))
				End If
				cs.Append(" ")
			Next
			lblStatus.Text = cs.PopAll
		Else
			lblStatus.Text = (CurrentSlideID + 1) & " of " & lstSlides.Size
		End If
	Else
		lblStatus.Text = "No images loaded"
	End If
	lblStatus.BringToFront
End Sub